package grammar;

import java.io.*;
import java_cup.runtime.*;

%%

%public
%type		Symbol
%char


%{
	private int lineNumber = 1;
	public int lineNumber() { return lineNumber; }

	Symbol token( int tokenType ) {
		return new Symbol( tokenType, yychar, yychar+yytext().length(), yytext() );
		}
%}

%init{
	yybegin( NORMAL );
%init}

space		=	[\ \t]
newline		=	\r|\n|\r\n
input		=	[^\r\n]
letter		=	[A-Za-z_]
octDigit	=	[0-7]
decDigit	=	[0-9]
hexDigit	=	[0-9A-Fa-f]
exponent	=	[eE][\+\-]?{decDigit}+
intsuff		=	([lL][uU]|[uU][lL]|[lL]|[uU])?
fltsuff		=	[fFlL]?
cchar		=	[^\'\\\r\n]|{escchar}
schar		=	[^\"\\\r\n]|{escchar}
escchar		=	\\([ntbrfva\\\'\"\?]|{octDigit}+|[xX]{hexDigit}+)
ident		=	{letter}({letter}|{decDigit})*
zero		=	0{intsuff}
octal		=	0{octDigit}+{intsuff}
decimal		=	[1-9]{decDigit}*{intsuff}
hexadecimal	=	0[xX]{hexDigit}+{intsuff}
float1		=	{decDigit}+\.{decDigit}+{fltsuff}
float2		=	{decDigit}+{exponent}{fltsuff}
float3		=	{decDigit}+\.{decDigit}+{exponent}{fltsuff}
character	=	[lL]?\'{cchar}+\'
string		=	[lL]?\"{schar}*\"

%state      NORMAL COMMENT

%%

{newline}		{ 
				lineNumber++;
				}
\\{newline}		{ 
				lineNumber++;
				}

<NORMAL> {
	{space}			{ }

	auto			{ return token( sym.AUTO ); }
	break			{ return token( sym.BREAK ); }
	case			{ return token( sym.CASE ); }
	char			{ return token( sym.CHAR ); }
	continue		{ return token( sym.CONTINUE ); }
	default			{ return token( sym.DEFAULT ); }
	do				{ return token( sym.DO ); }
	double			{ return token( sym.DOUBLE ); }
	else			{ return token( sym.ELSE ); }
	enum			{ return token( sym.ENUM ); }
	extern			{ return token( sym.EXTERN ); }
	float			{ return token( sym.FLOAT ); }
	for				{ return token( sym.FOR ); }
	goto			{ return token( sym.GOTO ); }
	if				{ return token( sym.IF ); }
	int				{ return token( sym.INT ); }
	long			{ return token( sym.LONG ); }
	register		{ return token( sym.REGISTER ); }
	return			{ return token( sym.RETURN ); }
	short			{ return token( sym.SHORT ); }
	signed			{ return token( sym.SIGNED ); }
	sizeof			{ return token( sym.SIZEOF ); }
	static			{ return token( sym.STATIC ); }
	struct			{ return token( sym.STRUCT ); }
	switch			{ return token( sym.SWITCH ); }
	typedef			{ return token( sym.TYPEDEF ); }
	union			{ return token( sym.UNION ); }
	unsigned		{ return token( sym.UNSIGNED ); }
	void			{ return token( sym.VOID ); }
	while			{ return token( sym.WHILE ); }

	";"				{ return token( sym.SEMICOLON ); }
	"..."			{ return token( sym.ETC ); }

	","				{ return token( sym.COMMA ); }

	"="				{ return token( sym.ASSIGN ); }
	"+="			{ return token( sym.PLUSASSIGN ); }
	"-="			{ return token( sym.MINUSASSIGN ); }
	"*="			{ return token( sym.TIMESASSIGN ); }
	"/="			{ return token( sym.DIVIDEASSIGN ); }
	"%="			{ return token( sym.MODASSIGN ); }
	"<<="			{ return token( sym.LEFTASSIGN ); }
	">>="			{ return token( sym.RIGHTASSIGN ); }
	"|="			{ return token( sym.BITORASSIGN ); }
	"^="			{ return token( sym.BITXORASSIGN ); }
	"&="			{ return token( sym.BITANDASSIGN ); }

	"?"				{ return token( sym.QUEST ); }
	":"				{ return token( sym.COLON ); }

	"||"			{ return token( sym.OR ); }

	"&&"			{ return token( sym.AND ); }

	"|"				{ return token( sym.BITOR ); }

	"^"				{ return token( sym.BITXOR ); }

	"=="			{ return token( sym.EQ ); }
	"!="			{ return token( sym.NE ); }

	"<"				{ return token( sym.LT ); }
	">"				{ return token( sym.GT ); }
	"<="			{ return token( sym.LE ); }
	">="			{ return token( sym.GE ); }

	"<<"			{ return token( sym.LEFTSHIFT ); }
	">>"			{ return token( sym.RIGHTSHIFT ); }

	"+"				{ return token( sym.PLUS ); }
	"-"				{ return token( sym.MINUS ); }

	"*"				{ return token( sym.STAR ); }
	"/"				{ return token( sym.DIVIDE ); }
	"%"				{ return token( sym.MOD ); }

	"++"			{ return token( sym.INCR ); }
	"--"			{ return token( sym.DECR ); }
	"&"				{ return token( sym.AMPERSAND ); }
	"!"				{ return token( sym.NOT ); }
	"~"				{ return token( sym.BITNOT ); }

	"."				{ return token( sym.DOT ); }
	"->"			{ return token( sym.ARROW ); }

	"("				{ return token( sym.LEFT ); }
	")"				{ return token( sym.RIGHT ); }

	"{"				{ return token( sym.LEFTCURLY ); }
	"}"				{ return token( sym.RIGHTCURLY ); }

	"["				{ return token( sym.LEFTSQ ); }
	"]"				{ return token( sym.RIGHTSQ ); }

	{ident}			{ return token( sym.IDENT ); }

	{zero}			{ return token( sym.INTCONST ); }
	{octal}			{ return token( sym.INTCONST ); }
	{decimal}		{ return token( sym.INTCONST ); }
	{hexadecimal}	{ return token( sym.INTCONST ); }

	{float1}		{ return token( sym.FLOATCONST ); }
	{float2}		{ return token( sym.FLOATCONST ); }
	{float3}		{ return token( sym.FLOATCONST ); }

	{character}		{ return token( sym.CHARCONST ); }
	{string}		{ return token( sym.STRINGCONST ); }

	"//"{input}*	{ }

	"/*"            { yybegin( COMMENT ); }
	.               { System.err.println( "Invalid character \"" + yytext() + "\"" ); }
	}
	
<COMMENT> {
	"*/"			{ yybegin( NORMAL ); }
	.				{ }
	}

<<EOF>>				{ return token( sym.EOF ); }

