package grammar;

import java.io.*;
import java_cup.runtime.*;

%%

%public
%type		Symbol
%char

%{
	private int lineNumber = 1;
	public int lineNumber() { return lineNumber; }
	
	public Symbol token( int tokenType ) {
		System.err.println( "Obtain token " + sym.terminal_name( tokenType ) 
			+ " \"" + yytext() + "\"" );
		return new Symbol( tokenType, yychar, yychar + yytext().length(), yytext() );
		}

%}

InputChar		=	[^\n\r]
SpaceChar		=	[\ \t]
LineChar		=	\n|\r|\r\n

Zero			=	0
DecInt			=	[1-9][0-9]*
OctalInt		=	0[0-7]+
HexInt			=	0[xX][0-9a-fA-F]+

Integer			=	( {Zero} | {DecInt} | {OctalInt} | {HexInt} )[lL]?
Exponent		=	[eE] [\+\-]? [0-9]+
Float1			=	[0-9]+ \. [0-9]+ {Exponent}?
Float2			=	\. [0-9]+ {Exponent}?
Float3			=	[0-9]+ \. {Exponent}?
Float4			=	[0-9]+ {Exponent}
Float			=	( {Float1} | {Float2} | {Float3} | {Float4} ) [fFdD]? | [0-9]+ [fFDd]
Ident			=	[A-Za-z_$] [A-Za-z_$0-9]*
CChar			=	[^\'\\\n\r] | {EscChar}
SChar			=	[^\"\\\n\r] | {EscChar}
EscChar			=	\\[ntbrf\\\'\"] | {OctalEscape}
OctalEscape		=	\\[0-7] | \\[0-7][0-7] | \\[0-3][0-7][0-7]

%%
abstract		{ return token( sym.ABSTRACT ); }
boolean			{ return token( sym.BOOLEAN ); }
break			{ return token( sym.BREAK ); }
byte			{ return token( sym.BYTE ); }
case			{ return token( sym.CASE ); }
catch			{ return token( sym.CATCH ); }
char			{ return token( sym.CHAR ); }
class			{ return token( sym.CLASS ); }
continue		{ return token( sym.CONTINUE ); }
default			{ return token( sym.DEFAULT ); }
do				{ return token( sym.DO ); }
double			{ return token( sym.DOUBLE ); }
else			{ return token( sym.ELSE ); }
extends			{ return token( sym.EXTENDS ); }
final			{ return token( sym.FINAL ); }
finally			{ return token( sym.FINALLY ); }
float			{ return token( sym.FLOAT ); }
for				{ return token( sym.FOR ); }
if				{ return token( sym.IF ); }
implements		{ return token( sym.IMPLEMENTS ); }
import			{ return token( sym.IMPORT ); }
instanceof		{ return token( sym.INSTANCEOF ); }
int				{ return token( sym.INT ); }
interface		{ return token( sym.INTERFACE ); }
long			{ return token( sym.LONG ); }
native			{ return token( sym.NATIVE ); }
new				{ return token( sym.NEW ); }
package			{ return token( sym.PACKAGE ); }
private			{ return token( sym.PRIVATE ); }
protected		{ return token( sym.PROTECTED ); }
public			{ return token( sym.PUBLIC ); }
return			{ return token( sym.RETURN ); }
short			{ return token( sym.SHORT ); }
static			{ return token( sym.STATIC ); }
super			{ return token( sym.SUPER ); }
switch			{ return token( sym.SWITCH ); }
synchronized	{ return token( sym.SYNCHRONIZED ); }
this			{ return token( sym.THIS ); }
throw			{ return token( sym.THROW ); }
throws			{ return token( sym.THROWS ); }
transient		{ return token( sym.TRANSIENT ); }
try				{ return token( sym.TRY ); }
void			{ return token( sym.VOID ); }
volatile		{ return token( sym.VOLATILE ); }
while			{ return token( sym.WHILE ); }

"("				{ return token( sym.LEFT ); }
")"				{ return token( sym.RIGHT ); }
"{"				{ return token( sym.LEFTCURLY ); }
"}"				{ return token( sym.RIGHTCURLY ); }
"["				{ return token( sym.LEFTSQ ); }
"]"				{ return token( sym.RIGHTSQ ); }

";"				{ return token( sym.SEMICOLON ); }
","				{ return token( sym.COMMA ); }
"."				{ return token( sym.DOT ); }

"="				{ return token( sym.ASSIGN ); }
"+="			{ return token( sym.PLUSASSIGN ); }
"-="			{ return token( sym.MINUSASSIGN ); }
"*="			{ return token( sym.STARASSIGN ); }
"/="			{ return token( sym.SLASHASSIGN ); }
"%="			{ return token( sym.PERCENTASSIGN ); }
"<<="			{ return token( sym.LEFTSHIFTASSIGN ); }
">>="			{ return token( sym.RIGHTSHIFTASSIGN ); }
">>>="			{ return token( sym.UNSIGNEDRIGHTSHIFTASSIGN ); }
"|="			{ return token( sym.BARASSIGN ); }
"^="			{ return token( sym.CARETASSIGN ); }
"&="			{ return token( sym.AMPERSANDASSIGN ); }

"?"				{ return token( sym.QUEST ); }
":"				{ return token( sym.COLON ); }

"||"			{ return token( sym.OR ); }
"&&"			{ return token( sym.AND ); }

"=="			{ return token( sym.EQ ); }
"!="			{ return token( sym.NE ); }
"<"				{ return token( sym.LT ); }
">"				{ return token( sym.GT ); }
"<="			{ return token( sym.LE ); }
">="			{ return token( sym.GE ); }

"<<"			{ return token( sym.LEFTSHIFT ); }
">>"			{ return token( sym.RIGHTSHIFT ); }
">>>"			{ return token( sym.UNSIGNEDRIGHTSHIFT ); }

"+"				{ return token( sym.PLUS ); }
"-"				{ return token( sym.MINUS ); }
"*"				{ return token( sym.STAR ); }
"/"				{ return token( sym.SLASH ); }
"%"				{ return token( sym.PERCENT ); }
"++"			{ return token( sym.INCR ); }
"--"			{ return token( sym.DECR ); }
"|"				{ return token( sym.BAR ); }
"^"				{ return token( sym.CARET ); }
"&"				{ return token( sym.AMPERSAND ); }
"!"				{ return token( sym.EXCLAIM ); }
"~"				{ return token( sym.TILDE ); }

true			{ return token( sym.BOOLEANLIT ); }
false			{ return token( sym.BOOLEANLIT ); }
null			{ return token( sym.NULLLIT ); }

{Integer}		{ return token( sym.INTEGERLIT ); }

{Float}			{ return token( sym.FLOATLIT ); }

\'{CChar}\'		{ return token( sym.CHARLIT ); }
\"{SChar}*\"	{return token( sym.STRINGLIT ); }

{Ident}			{ return token( sym.IDENT ); }

"//"{InputChar}*	{ }

"/*"~"*/"		{ }

{LineChar}		{ lineNumber++; }
{SpaceChar}		{ }
.				{ return token( sym.error ); }
<<EOF>>			{ return token( sym.EOF ); }

