package grammar;

import java.util.*;

%%

%public
%type		void
%char

%{
	final static String leftParenth = "(";
	final static String leftBrace = "{";
	final static String leftBracket = "[";
	
	int line = 1;

	class IdentInfo {
		private String ident;
		public String ident() { return ident; }
		
		private Vector< Integer > lines = new Vector< Integer >();
		
		public IdentInfo( String ident ) {
			this.ident = ident;
			}
		
		void addLine( int line ) {
			lines.addElement( new Integer( line ) );
			}
		
		void print() {
			System.out.print( ident + ":" );
			for ( int i = 0; i < lines.size(); i++ )
				System.out.print( "\t" + lines.elementAt( i ) );
			System.out.println();
			}
		}
		
	class IdentTable {		
		Vector< IdentInfo > table = new Vector< IdentInfo >();
	
		public IdentInfo identInfo( int i ) {
			return ( IdentInfo ) table.elementAt( i );
			}

		public int size() { 
			return table.size();
			}
		
		void addLine( String ident, int line ) {
			//	System.out.println( "addLine( " + ident + ", " + line + " )" );
			for ( int i = 0; i < size(); i++ ) {
				IdentInfo identInfo = identInfo( i );
				int compare = identInfo.ident().compareTo( ident );
				if ( compare == 0 ) {
					identInfo.addLine( line );
					return;
					}
				else if ( compare > 0 ) {
					identInfo = new IdentInfo( ident );
					table.insertElementAt( identInfo, i );
					identInfo.addLine( line );
					return;
					}
				}
					
			IdentInfo identInfo = new IdentInfo( ident );
			table.insertElementAt( identInfo, size() );
			identInfo.addLine( line );
			}
			
		void print() {
			System.out.println( "Identifier Table" );
			for ( int i = 0; i < size(); i++ ) {
				IdentInfo identInfo = identInfo( i );
				identInfo.print();
				}
			}
		}
	
	IdentTable identTable = new IdentTable();
	
	void error( String s ) {
		System.err.println( line + ": " + s );
		}
		
	Stack< String > stack = new Stack< String >();
	
	void push( String symbol ) {
		stack.push( symbol );
		}
	
	void pop( String left ) {
		while ( ! stack.empty() ) {
			String top = ( String ) stack.pop();
			if ( top.equals( left ) )
				return;
			error( "Incomplete " + top + "..." );
			}
		error( "Missing " + left + "..." );
		}
	
	public void finish() {
		while ( ! stack.empty() ) {
			String top = ( String ) stack.pop();
			error( "Incomplete " + top + "..." );
			}
		if ( yystate() != NORMAL )
			error( "Incomplete /* ... " );
		identTable.print();
		}		
%}

%init{
	yybegin( NORMAL );
%init}

%eofval{
	finish();
	return;
%eofval}

InputChar		=	[^\n\r]
SpaceChar		=	[\ \t]
LineChar		=	\n|\r|\r\n

Zero			=	0
DecInt			=	([1-9][0-9]*)
OctalInt		=	(0[0-7]+)
HexInt			=	(0[xX][0-9a-fA-F]+)

Integer			=	(({Zero}|{DecInt}|{OctalInt}|{HexInt})[lL]?)
Exponent		=	([eE][\+\-]?[0-9]+)
Float1			=	([0-9]+\.[0-9]+{Exponent}?)
Float2			=	(\.[0-9]+{Exponent}?)
Float3			=	([0-9]+\.{Exponent}?)
Float4			=	([0-9]+{Exponent})
Float			=	(({Float1}|{Float2}|{Float3}|{Float4})[fFdD]?|[0-9]+[fFDd])
Ident			=	([A-Za-z_$][A-Za-z_$0-9]*)
CChar			=	([^\'\\\n\r]|{EscChar})
SChar			=	([^\"\\\n\r]|{EscChar})
EscChar			=	(\\[ntbrf\\\'\"]|{OctalEscape})
OctalEscape		=	(\\[0-7]|\\[0-7][0-7]|\\[0-3][0-7][0-7])

%state NORMAL, COMMENT

%%

<NORMAL> {
	abstract		{ }
	boolean			{ }
	break			{ }
	byte			{ }
	case			{ }
	catch			{ }
	char			{ }
	class			{ }
	continue		{ }
	default			{ }
	do				{ }
	double			{ }
	else			{ }
	extends			{ }
	final			{ }
	finally			{ }
	float			{ }
	for				{ }
	if				{ }
	implements		{ }
	import			{ }
	instanceof		{ }
	int				{ }
	interface		{ }
	long			{ }
	native			{ }
	new				{ }
	package			{ }
	private			{ }
	protected		{ }
	public			{ }
	return			{ }
	short			{ }
	static			{ }
	super			{ }
	switch			{ }
	synchronized	{ }
	this			{ }
	throw			{ }
	throws			{ }
	transient		{ }
	try				{ }
	void			{ }
	volatile		{ }
	while			{ }
	
	"("				{ push( leftParenth ); }
	")"				{ pop( leftParenth ); }
	"{"				{ push( leftBrace ); }
	"}"				{ pop( leftBrace ); }
	"["				{ push( leftBracket ); }
	"]"				{ pop( leftBracket ); }
	
	";"				{ }
	","				{ }
	"."				{ }
	
	"="				{ }
	"+="			{ }
	"-="			{ }
	"*="			{ }
	"/="			{ }
	"%="			{ }
	"<<="			{ }
	">>="			{ }
	">>>="			{ }
	"|="			{ }
	"^="			{ }
	"&="			{ }
	
	"?"				{ }
	":"				{ }
	
	"||"			{ }
	"&&"			{ }
	
	"=="			{ }
	"!="			{ }
	"<"				{ }
	">"				{ }
	"<="			{ }
	">="			{ }
	
	"<<"			{ }
	">>"			{ }
	">>>"			{ }
	
	"+"				{ }
	"-"				{ }
	"*"				{ }
	"/"				{ }
	"%"				{ }
	"++"			{ }
	"--"			{ }
	"|"				{ }
	"^"				{ }
	"&"				{ }
	"!"				{ }
	"~"				{ }
	
	true			{ }
	false			{ }
	null			{ }
	
	{Integer}		{ }
	
	{Float}			{ }
	
	\'{CChar}\'		{ }
	\"{SChar}*\"	{ }
	
	{Ident}			{ identTable.addLine( yytext(), line ); }
	
	"//"{InputChar}*	{ }
	
	"/*"				{ yybegin( COMMENT ); }
	"*/"				{ error( "Missing /* ..." ); }
	
	{LineChar}			{ line++; }
	{SpaceChar}			{ }
	
	.					{ }
	}

<COMMENT> {
	"/*"				{ error( "Nested /* ... " ); }
	{InputChar}		{ }
	{LineChar}			{ line++; }
	"*/"				{ yybegin( NORMAL ); }
	}
